%%
%%  domain.pl:  Axiomatisation of the "Cooking kitchenhands" domain for MIndiGolog
%%		with location and is_nearest agent resource allocation algorithm


%%  
%%  kitchenhand(Kh):  	specify kitchenhands in the system, incharge of cleaning, preparing raw material, cleaning utensil
%%  chef(Chef):		specify the chef in the system, inchage of cooking dish
%%
kitchenhand(kh01).
kitchenhand(kh02).
kitchenhand(kh03).
kitchenhand(kh04).
kitchenhand(kh05).
chef(chef01).


%% 
%%  task(T):  specify the tasks that can be performed
%%
%%  This predicate is true when T is a task that the kitchenhands in the system
%%  can perform.  Tasks are typically parameterised in terms of the
%%  objects on which they operate and where they operate.
%%

%%  mix(Uten,Dur):  mix the contents of utensil Uten for duration Dur
task(mix(Uten,_)) :- 
    obj_is_type(Uten,utensil).

%%  marinate(Uten,Dur):  marinate the contents of utensil Uten for duration Dur
task(marinate(Uten,_)) :- 
    obj_is_type(Uten,utensil).

%%  cut(Uten,Tb):  cut the contents of utensil Uten on table Tb
task(cut(Uten,Tb)) :-
    obj_is_type(Uten,utensil),obj_is_type(Tb,table).

%%  clean(Obj,Sin):  clean the object Obj in sink Sin, Obj can be utensil or raw material
task(clean(Obj,Sin)) :-
    obj_is_type(Obj,needcleanstuff),obj_is_type(Sin,sink).

%% goto(A,B):	go from block A to block B
task(goto(A,B)) :-
    obj_is_type(A,block),obj_is_type(B,block).

%%
%%  task_duration(Kh,Task,Dur):  specify duration of tasks
%%
%%  This predicate is true when Dur is the time taken by kitchenhand Kh
%%  to perform the task Task.
%%

task_duration(_,mix(_,D),D).
task_duration(_,marinate(_,D),D).
task_duration(_,cut(_,_),D) :-
    (
        D = 3
    ).

task_duration(_,clean(_,_),D) :-
    (
        D = 1
    ).

%% there is a hash table to determain the duration of goto proc.
task_duration(_,goto(A,B),D) :-
(
	(	
		A=block00,B=block01 -> D = 1;
		A=block00,B=block02 -> D = 2;
		A=block00,B=block03 -> D = 3;
		A=block00,B=block04 -> D = 4;
		A=block00,B=block10 -> D = 1;
		A=block00,B=block11 -> D = 2;
		A=block00,B=block12 -> D = 3;
		A=block00,B=block13 -> D = 4;
		A=block00,B=block14 -> D = 5;
		A=block00,B=block20 -> D = 2;
		A=block00,B=block21 -> D = 3;
		A=block00,B=block22 -> D = 4;
		A=block00,B=block23 -> D = 5;
		A=block00,B=block24 -> D = 6;
		A=block00,B=block30 -> D = 3;
		A=block00,B=block31 -> D = 4;
		A=block00,B=block32 -> D = 5;
		A=block00,B=block33 -> D = 6;
		A=block00,B=block34 -> D = 7;
		A=block00,B=block40 -> D = 4;
		A=block00,B=block41 -> D = 5;
		A=block00,B=block42 -> D = 6;
		A=block00,B=block43 -> D = 7;
		A=block00,B=block44 -> D = 8;
		A=block01,B=block00 -> D = 1;
		A=block01,B=block02 -> D = 1;
		A=block01,B=block03 -> D = 2;
		A=block01,B=block04 -> D = 3;
		A=block01,B=block10 -> D = 2;
		A=block01,B=block11 -> D = 1;
		A=block01,B=block12 -> D = 2;
		A=block01,B=block13 -> D = 3;
		A=block01,B=block14 -> D = 4;
		A=block01,B=block20 -> D = 3;
		A=block01,B=block21 -> D = 2;
		A=block01,B=block22 -> D = 3;
		A=block01,B=block23 -> D = 4;
		A=block01,B=block24 -> D = 5;
		A=block01,B=block30 -> D = 4;
		A=block01,B=block31 -> D = 3;
		A=block01,B=block32 -> D = 4;
		A=block01,B=block33 -> D = 5;
		A=block01,B=block34 -> D = 6;
		A=block01,B=block40 -> D = 5;
		A=block01,B=block41 -> D = 4;
		A=block01,B=block42 -> D = 5;
		A=block01,B=block43 -> D = 6;
		A=block01,B=block44 -> D = 7;
		A=block02,B=block00 -> D = 2;
		A=block02,B=block01 -> D = 1;
		A=block02,B=block03 -> D = 1;
		A=block02,B=block04 -> D = 2;
		A=block02,B=block10 -> D = 3;
		A=block02,B=block11 -> D = 2;
		A=block02,B=block12 -> D = 1;
		A=block02,B=block13 -> D = 2;
		A=block02,B=block14 -> D = 3;
		A=block02,B=block20 -> D = 4;
		A=block02,B=block21 -> D = 3;
		A=block02,B=block22 -> D = 2;
		A=block02,B=block23 -> D = 3;
		A=block02,B=block24 -> D = 4;
		A=block02,B=block30 -> D = 5;
		A=block02,B=block31 -> D = 4;
		A=block02,B=block32 -> D = 3;
		A=block02,B=block33 -> D = 4;
		A=block02,B=block34 -> D = 5;
		A=block02,B=block40 -> D = 6;
		A=block02,B=block41 -> D = 5;
		A=block02,B=block42 -> D = 4;
		A=block02,B=block43 -> D = 5;
		A=block02,B=block44 -> D = 6;
		A=block03,B=block00 -> D = 3;
		A=block03,B=block01 -> D = 2;
		A=block03,B=block02 -> D = 1;
		A=block03,B=block04 -> D = 1;
		A=block03,B=block10 -> D = 4;
		A=block03,B=block11 -> D = 3;
		A=block03,B=block12 -> D = 2;
		A=block03,B=block13 -> D = 1;
		A=block03,B=block14 -> D = 2;
		A=block03,B=block20 -> D = 5;
		A=block03,B=block21 -> D = 4;
		A=block03,B=block22 -> D = 3;
		A=block03,B=block23 -> D = 2;
		A=block03,B=block24 -> D = 3;
		A=block03,B=block30 -> D = 6;
		A=block03,B=block31 -> D = 5;
		A=block03,B=block32 -> D = 4;
		A=block03,B=block33 -> D = 3;
		A=block03,B=block34 -> D = 4;
		A=block03,B=block40 -> D = 7;
		A=block03,B=block41 -> D = 6;
		A=block03,B=block42 -> D = 5;
		A=block03,B=block43 -> D = 4;
		A=block03,B=block44 -> D = 5;
		A=block04,B=block00 -> D = 4;
		A=block04,B=block01 -> D = 3;
		A=block04,B=block02 -> D = 2;
		A=block04,B=block03 -> D = 1;
		A=block04,B=block10 -> D = 5;
		A=block04,B=block11 -> D = 4;
		A=block04,B=block12 -> D = 3;
		A=block04,B=block13 -> D = 2;
		A=block04,B=block14 -> D = 1;
		A=block04,B=block20 -> D = 6;
		A=block04,B=block21 -> D = 5;
		A=block04,B=block22 -> D = 4;
		A=block04,B=block23 -> D = 3;
		A=block04,B=block24 -> D = 2;
		A=block04,B=block30 -> D = 7;
		A=block04,B=block31 -> D = 6;
		A=block04,B=block32 -> D = 5;
		A=block04,B=block33 -> D = 4;
		A=block04,B=block34 -> D = 3;
		A=block04,B=block40 -> D = 8;
		A=block04,B=block41 -> D = 7;
		A=block04,B=block42 -> D = 6;
		A=block04,B=block43 -> D = 5;
		A=block04,B=block44 -> D = 4;
		A=block10,B=block00 -> D = 1;
		A=block10,B=block01 -> D = 2;
		A=block10,B=block02 -> D = 3;
		A=block10,B=block03 -> D = 4;
		A=block10,B=block04 -> D = 5;
		A=block10,B=block11 -> D = 1;
		A=block10,B=block12 -> D = 2;
		A=block10,B=block13 -> D = 3;
		A=block10,B=block14 -> D = 4;
		A=block10,B=block20 -> D = 1;
		A=block10,B=block21 -> D = 2;
		A=block10,B=block22 -> D = 3;
		A=block10,B=block23 -> D = 4;
		A=block10,B=block24 -> D = 5;
		A=block10,B=block30 -> D = 2;
		A=block10,B=block31 -> D = 3;
		A=block10,B=block32 -> D = 4;
		A=block10,B=block33 -> D = 5;
		A=block10,B=block34 -> D = 6;
		A=block10,B=block40 -> D = 3;
		A=block10,B=block41 -> D = 4;
		A=block10,B=block42 -> D = 5;
		A=block10,B=block43 -> D = 6;
		A=block10,B=block44 -> D = 7;
		A=block11,B=block00 -> D = 2;
		A=block11,B=block01 -> D = 1;
		A=block11,B=block02 -> D = 2;
		A=block11,B=block03 -> D = 3;
		A=block11,B=block04 -> D = 4;
		A=block11,B=block10 -> D = 1;
		A=block11,B=block12 -> D = 1;
		A=block11,B=block13 -> D = 2;
		A=block11,B=block14 -> D = 3;
		A=block11,B=block20 -> D = 2;
		A=block11,B=block21 -> D = 1;
		A=block11,B=block22 -> D = 2;
		A=block11,B=block23 -> D = 3;
		A=block11,B=block24 -> D = 4;
		A=block11,B=block30 -> D = 3;
		A=block11,B=block31 -> D = 2;
		A=block11,B=block32 -> D = 3;
		A=block11,B=block33 -> D = 4;
		A=block11,B=block34 -> D = 5;
		A=block11,B=block40 -> D = 4;
		A=block11,B=block41 -> D = 3;
		A=block11,B=block42 -> D = 4;
		A=block11,B=block43 -> D = 5;
		A=block11,B=block44 -> D = 6;
		A=block12,B=block00 -> D = 3;
		A=block12,B=block01 -> D = 2;
		A=block12,B=block02 -> D = 1;
		A=block12,B=block03 -> D = 2;
		A=block12,B=block04 -> D = 3;
		A=block12,B=block10 -> D = 2;
		A=block12,B=block11 -> D = 1;
		A=block12,B=block13 -> D = 1;
		A=block12,B=block14 -> D = 2;
		A=block12,B=block20 -> D = 3;
		A=block12,B=block21 -> D = 2;
		A=block12,B=block22 -> D = 1;
		A=block12,B=block23 -> D = 2;
		A=block12,B=block24 -> D = 3;
		A=block12,B=block30 -> D = 4;
		A=block12,B=block31 -> D = 3;
		A=block12,B=block32 -> D = 2;
		A=block12,B=block33 -> D = 3;
		A=block12,B=block34 -> D = 4;
		A=block12,B=block40 -> D = 5;
		A=block12,B=block41 -> D = 4;
		A=block12,B=block42 -> D = 3;
		A=block12,B=block43 -> D = 4;
		A=block12,B=block44 -> D = 5;
		A=block13,B=block00 -> D = 4;
		A=block13,B=block01 -> D = 3;
		A=block13,B=block02 -> D = 2;
		A=block13,B=block03 -> D = 1;
		A=block13,B=block04 -> D = 2;
		A=block13,B=block10 -> D = 3;
		A=block13,B=block11 -> D = 2;
		A=block13,B=block12 -> D = 1;
		A=block13,B=block14 -> D = 1;
		A=block13,B=block20 -> D = 4;
		A=block13,B=block21 -> D = 3;
		A=block13,B=block22 -> D = 2;
		A=block13,B=block23 -> D = 1;
		A=block13,B=block24 -> D = 2;
		A=block13,B=block30 -> D = 5;
		A=block13,B=block31 -> D = 4;
		A=block13,B=block32 -> D = 3;
		A=block13,B=block33 -> D = 2;
		A=block13,B=block34 -> D = 3;
		A=block13,B=block40 -> D = 6;
		A=block13,B=block41 -> D = 5;
		A=block13,B=block42 -> D = 4;
		A=block13,B=block43 -> D = 3;
		A=block13,B=block44 -> D = 4;
		A=block14,B=block00 -> D = 5;
		A=block14,B=block01 -> D = 4;
		A=block14,B=block02 -> D = 3;
		A=block14,B=block03 -> D = 2;
		A=block14,B=block04 -> D = 1;
		A=block14,B=block10 -> D = 4;
		A=block14,B=block11 -> D = 3;
		A=block14,B=block12 -> D = 2;
		A=block14,B=block13 -> D = 1;
		A=block14,B=block20 -> D = 5;
		A=block14,B=block21 -> D = 4;
		A=block14,B=block22 -> D = 3;
		A=block14,B=block23 -> D = 2;
		A=block14,B=block24 -> D = 1;
		A=block14,B=block30 -> D = 6;
		A=block14,B=block31 -> D = 5;
		A=block14,B=block32 -> D = 4;
		A=block14,B=block33 -> D = 3;
		A=block14,B=block34 -> D = 2;
		A=block14,B=block40 -> D = 7;
		A=block14,B=block41 -> D = 6;
		A=block14,B=block42 -> D = 5;
		A=block14,B=block43 -> D = 4;
		A=block14,B=block44 -> D = 3;
		A=block20,B=block00 -> D = 2;
		A=block20,B=block01 -> D = 3;
		A=block20,B=block02 -> D = 4;
		A=block20,B=block03 -> D = 5;
		A=block20,B=block04 -> D = 6;
		A=block20,B=block10 -> D = 1;
		A=block20,B=block11 -> D = 2;
		A=block20,B=block12 -> D = 3;
		A=block20,B=block13 -> D = 4;
		A=block20,B=block14 -> D = 5;
		A=block20,B=block21 -> D = 1;
		A=block20,B=block22 -> D = 2;
		A=block20,B=block23 -> D = 3;
		A=block20,B=block24 -> D = 4;
		A=block20,B=block30 -> D = 1;
		A=block20,B=block31 -> D = 2;
		A=block20,B=block32 -> D = 3;
		A=block20,B=block33 -> D = 4;
		A=block20,B=block34 -> D = 5;
		A=block20,B=block40 -> D = 2;
		A=block20,B=block41 -> D = 3;
		A=block20,B=block42 -> D = 4;
		A=block20,B=block43 -> D = 5;
		A=block20,B=block44 -> D = 6;
		A=block21,B=block00 -> D = 3;
		A=block21,B=block01 -> D = 2;
		A=block21,B=block02 -> D = 3;
		A=block21,B=block03 -> D = 4;
		A=block21,B=block04 -> D = 5;
		A=block21,B=block10 -> D = 2;
		A=block21,B=block11 -> D = 1;
		A=block21,B=block12 -> D = 2;
		A=block21,B=block13 -> D = 3;
		A=block21,B=block14 -> D = 4;
		A=block21,B=block20 -> D = 1;
		A=block21,B=block22 -> D = 1;
		A=block21,B=block23 -> D = 2;
		A=block21,B=block24 -> D = 3;
		A=block21,B=block30 -> D = 2;
		A=block21,B=block31 -> D = 1;
		A=block21,B=block32 -> D = 2;
		A=block21,B=block33 -> D = 3;
		A=block21,B=block34 -> D = 4;
		A=block21,B=block40 -> D = 3;
		A=block21,B=block41 -> D = 2;
		A=block21,B=block42 -> D = 3;
		A=block21,B=block43 -> D = 4;
		A=block21,B=block44 -> D = 5;
		A=block22,B=block00 -> D = 4;
		A=block22,B=block01 -> D = 3;
		A=block22,B=block02 -> D = 2;
		A=block22,B=block03 -> D = 3;
		A=block22,B=block04 -> D = 4;
		A=block22,B=block10 -> D = 3;
		A=block22,B=block11 -> D = 2;
		A=block22,B=block12 -> D = 1;
		A=block22,B=block13 -> D = 2;
		A=block22,B=block14 -> D = 3;
		A=block22,B=block20 -> D = 2;
		A=block22,B=block21 -> D = 1;
		A=block22,B=block23 -> D = 1;
		A=block22,B=block24 -> D = 2;
		A=block22,B=block30 -> D = 3;
		A=block22,B=block31 -> D = 2;
		A=block22,B=block32 -> D = 1;
		A=block22,B=block33 -> D = 2;
		A=block22,B=block34 -> D = 3;
		A=block22,B=block40 -> D = 4;
		A=block22,B=block41 -> D = 3;
		A=block22,B=block42 -> D = 2;
		A=block22,B=block43 -> D = 3;
		A=block22,B=block44 -> D = 4;
		A=block23,B=block00 -> D = 5;
		A=block23,B=block01 -> D = 4;
		A=block23,B=block02 -> D = 3;
		A=block23,B=block03 -> D = 2;
		A=block23,B=block04 -> D = 3;
		A=block23,B=block10 -> D = 4;
		A=block23,B=block11 -> D = 3;
		A=block23,B=block12 -> D = 2;
		A=block23,B=block13 -> D = 1;
		A=block23,B=block14 -> D = 2;
		A=block23,B=block20 -> D = 3;
		A=block23,B=block21 -> D = 2;
		A=block23,B=block22 -> D = 1;
		A=block23,B=block24 -> D = 1;
		A=block23,B=block30 -> D = 4;
		A=block23,B=block31 -> D = 3;
		A=block23,B=block32 -> D = 2;
		A=block23,B=block33 -> D = 1;
		A=block23,B=block34 -> D = 2;
		A=block23,B=block40 -> D = 5;
		A=block23,B=block41 -> D = 4;
		A=block23,B=block42 -> D = 3;
		A=block23,B=block43 -> D = 2;
		A=block23,B=block44 -> D = 3;
		A=block24,B=block00 -> D = 6;
		A=block24,B=block01 -> D = 5;
		A=block24,B=block02 -> D = 4;
		A=block24,B=block03 -> D = 3;
		A=block24,B=block04 -> D = 2;
		A=block24,B=block10 -> D = 5;
		A=block24,B=block11 -> D = 4;
		A=block24,B=block12 -> D = 3;
		A=block24,B=block13 -> D = 2;
		A=block24,B=block14 -> D = 1;
		A=block24,B=block20 -> D = 4;
		A=block24,B=block21 -> D = 3;
		A=block24,B=block22 -> D = 2;
		A=block24,B=block23 -> D = 1;
		A=block24,B=block30 -> D = 5;
		A=block24,B=block31 -> D = 4;
		A=block24,B=block32 -> D = 3;
		A=block24,B=block33 -> D = 2;
		A=block24,B=block34 -> D = 1;
		A=block24,B=block40 -> D = 6;
		A=block24,B=block41 -> D = 5;
		A=block24,B=block42 -> D = 4;
		A=block24,B=block43 -> D = 3;
		A=block24,B=block44 -> D = 2;
		A=block30,B=block00 -> D = 3;
		A=block30,B=block01 -> D = 4;
		A=block30,B=block02 -> D = 5;
		A=block30,B=block03 -> D = 6;
		A=block30,B=block04 -> D = 7;
		A=block30,B=block10 -> D = 2;
		A=block30,B=block11 -> D = 3;
		A=block30,B=block12 -> D = 4;
		A=block30,B=block13 -> D = 5;
		A=block30,B=block14 -> D = 6;
		A=block30,B=block20 -> D = 1;
		A=block30,B=block21 -> D = 2;
		A=block30,B=block22 -> D = 3;
		A=block30,B=block23 -> D = 4;
		A=block30,B=block24 -> D = 5;
		A=block30,B=block31 -> D = 1;
		A=block30,B=block32 -> D = 2;
		A=block30,B=block33 -> D = 3;
		A=block30,B=block34 -> D = 4;
		A=block30,B=block40 -> D = 1;
		A=block30,B=block41 -> D = 2;
		A=block30,B=block42 -> D = 3;
		A=block30,B=block43 -> D = 4;
		A=block30,B=block44 -> D = 5;
		A=block31,B=block00 -> D = 4;
		A=block31,B=block01 -> D = 3;
		A=block31,B=block02 -> D = 4;
		A=block31,B=block03 -> D = 5;
		A=block31,B=block04 -> D = 6;
		A=block31,B=block10 -> D = 3;
		A=block31,B=block11 -> D = 2;
		A=block31,B=block12 -> D = 3;
		A=block31,B=block13 -> D = 4;
		A=block31,B=block14 -> D = 5;
		A=block31,B=block20 -> D = 2;
		A=block31,B=block21 -> D = 1;
		A=block31,B=block22 -> D = 2;
		A=block31,B=block23 -> D = 3;
		A=block31,B=block24 -> D = 4;
		A=block31,B=block30 -> D = 1;
		A=block31,B=block32 -> D = 1;
		A=block31,B=block33 -> D = 2;
		A=block31,B=block34 -> D = 3;
		A=block31,B=block40 -> D = 2;
		A=block31,B=block41 -> D = 1;
		A=block31,B=block42 -> D = 2;
		A=block31,B=block43 -> D = 3;
		A=block31,B=block44 -> D = 4;
		A=block32,B=block00 -> D = 5;
		A=block32,B=block01 -> D = 4;
		A=block32,B=block02 -> D = 3;
		A=block32,B=block03 -> D = 4;
		A=block32,B=block04 -> D = 5;
		A=block32,B=block10 -> D = 4;
		A=block32,B=block11 -> D = 3;
		A=block32,B=block12 -> D = 2;
		A=block32,B=block13 -> D = 3;
		A=block32,B=block14 -> D = 4;
		A=block32,B=block20 -> D = 3;
		A=block32,B=block21 -> D = 2;
		A=block32,B=block22 -> D = 1;
		A=block32,B=block23 -> D = 2;
		A=block32,B=block24 -> D = 3;
		A=block32,B=block30 -> D = 2;
		A=block32,B=block31 -> D = 1;
		A=block32,B=block33 -> D = 1;
		A=block32,B=block34 -> D = 2;
		A=block32,B=block40 -> D = 3;
		A=block32,B=block41 -> D = 2;
		A=block32,B=block42 -> D = 1;
		A=block32,B=block43 -> D = 2;
		A=block32,B=block44 -> D = 3;
		A=block33,B=block00 -> D = 6;
		A=block33,B=block01 -> D = 5;
		A=block33,B=block02 -> D = 4;
		A=block33,B=block03 -> D = 3;
		A=block33,B=block04 -> D = 4;
		A=block33,B=block10 -> D = 5;
		A=block33,B=block11 -> D = 4;
		A=block33,B=block12 -> D = 3;
		A=block33,B=block13 -> D = 2;
		A=block33,B=block14 -> D = 3;
		A=block33,B=block20 -> D = 4;
		A=block33,B=block21 -> D = 3;
		A=block33,B=block22 -> D = 2;
		A=block33,B=block23 -> D = 1;
		A=block33,B=block24 -> D = 2;
		A=block33,B=block30 -> D = 3;
		A=block33,B=block31 -> D = 2;
		A=block33,B=block32 -> D = 1;
		A=block33,B=block34 -> D = 1;
		A=block33,B=block40 -> D = 4;
		A=block33,B=block41 -> D = 3;
		A=block33,B=block42 -> D = 2;
		A=block33,B=block43 -> D = 1;
		A=block33,B=block44 -> D = 2;
		A=block34,B=block00 -> D = 7;
		A=block34,B=block01 -> D = 6;
		A=block34,B=block02 -> D = 5;
		A=block34,B=block03 -> D = 4;
		A=block34,B=block04 -> D = 3;
		A=block34,B=block10 -> D = 6;
		A=block34,B=block11 -> D = 5;
		A=block34,B=block12 -> D = 4;
		A=block34,B=block13 -> D = 3;
		A=block34,B=block14 -> D = 2;
		A=block34,B=block20 -> D = 5;
		A=block34,B=block21 -> D = 4;
		A=block34,B=block22 -> D = 3;
		A=block34,B=block23 -> D = 2;
		A=block34,B=block24 -> D = 1;
		A=block34,B=block30 -> D = 4;
		A=block34,B=block31 -> D = 3;
		A=block34,B=block32 -> D = 2;
		A=block34,B=block33 -> D = 1;
		A=block34,B=block40 -> D = 5;
		A=block34,B=block41 -> D = 4;
		A=block34,B=block42 -> D = 3;
		A=block34,B=block43 -> D = 2;
		A=block34,B=block44 -> D = 1;
		A=block40,B=block00 -> D = 4;
		A=block40,B=block01 -> D = 5;
		A=block40,B=block02 -> D = 6;
		A=block40,B=block03 -> D = 7;
		A=block40,B=block04 -> D = 8;
		A=block40,B=block10 -> D = 3;
		A=block40,B=block11 -> D = 4;
		A=block40,B=block12 -> D = 5;
		A=block40,B=block13 -> D = 6;
		A=block40,B=block14 -> D = 7;
		A=block40,B=block20 -> D = 2;
		A=block40,B=block21 -> D = 3;
		A=block40,B=block22 -> D = 4;
		A=block40,B=block23 -> D = 5;
		A=block40,B=block24 -> D = 6;
		A=block40,B=block30 -> D = 1;
		A=block40,B=block31 -> D = 2;
		A=block40,B=block32 -> D = 3;
		A=block40,B=block33 -> D = 4;
		A=block40,B=block34 -> D = 5;
		A=block40,B=block41 -> D = 1;
		A=block40,B=block42 -> D = 2;
		A=block40,B=block43 -> D = 3;
		A=block40,B=block44 -> D = 4;
		A=block41,B=block00 -> D = 5;
		A=block41,B=block01 -> D = 4;
		A=block41,B=block02 -> D = 5;
		A=block41,B=block03 -> D = 6;
		A=block41,B=block04 -> D = 7;
		A=block41,B=block10 -> D = 4;
		A=block41,B=block11 -> D = 3;
		A=block41,B=block12 -> D = 4;
		A=block41,B=block13 -> D = 5;
		A=block41,B=block14 -> D = 6;
		A=block41,B=block20 -> D = 3;
		A=block41,B=block21 -> D = 2;
		A=block41,B=block22 -> D = 3;
		A=block41,B=block23 -> D = 4;
		A=block41,B=block24 -> D = 5;
		A=block41,B=block30 -> D = 2;
		A=block41,B=block31 -> D = 1;
		A=block41,B=block32 -> D = 2;
		A=block41,B=block33 -> D = 3;
		A=block41,B=block34 -> D = 4;
		A=block41,B=block40 -> D = 1;
		A=block41,B=block42 -> D = 1;
		A=block41,B=block43 -> D = 2;
		A=block41,B=block44 -> D = 3;
		A=block42,B=block00 -> D = 6;
		A=block42,B=block01 -> D = 5;
		A=block42,B=block02 -> D = 4;
		A=block42,B=block03 -> D = 5;
		A=block42,B=block04 -> D = 6;
		A=block42,B=block10 -> D = 5;
		A=block42,B=block11 -> D = 4;
		A=block42,B=block12 -> D = 3;
		A=block42,B=block13 -> D = 4;
		A=block42,B=block14 -> D = 5;
		A=block42,B=block20 -> D = 4;
		A=block42,B=block21 -> D = 3;
		A=block42,B=block22 -> D = 2;
		A=block42,B=block23 -> D = 3;
		A=block42,B=block24 -> D = 4;
		A=block42,B=block30 -> D = 3;
		A=block42,B=block31 -> D = 2;
		A=block42,B=block32 -> D = 1;
		A=block42,B=block33 -> D = 2;
		A=block42,B=block34 -> D = 3;
		A=block42,B=block40 -> D = 2;
		A=block42,B=block41 -> D = 1;
		A=block42,B=block43 -> D = 1;
		A=block42,B=block44 -> D = 2;
		A=block43,B=block00 -> D = 7;
		A=block43,B=block01 -> D = 6;
		A=block43,B=block02 -> D = 5;
		A=block43,B=block03 -> D = 4;
		A=block43,B=block04 -> D = 5;
		A=block43,B=block10 -> D = 6;
		A=block43,B=block11 -> D = 5;
		A=block43,B=block12 -> D = 4;
		A=block43,B=block13 -> D = 3;
		A=block43,B=block14 -> D = 4;
		A=block43,B=block20 -> D = 5;
		A=block43,B=block21 -> D = 4;
		A=block43,B=block22 -> D = 3;
		A=block43,B=block23 -> D = 2;
		A=block43,B=block24 -> D = 3;
		A=block43,B=block30 -> D = 4;
		A=block43,B=block31 -> D = 3;
		A=block43,B=block32 -> D = 2;
		A=block43,B=block33 -> D = 1;
		A=block43,B=block34 -> D = 2;
		A=block43,B=block40 -> D = 3;
		A=block43,B=block41 -> D = 2;
		A=block43,B=block42 -> D = 1;
		A=block43,B=block44 -> D = 1;
		A=block44,B=block00 -> D = 8;
		A=block44,B=block01 -> D = 7;
		A=block44,B=block02 -> D = 6;
		A=block44,B=block03 -> D = 5;
		A=block44,B=block04 -> D = 4;
		A=block44,B=block10 -> D = 7;
		A=block44,B=block11 -> D = 6;
		A=block44,B=block12 -> D = 5;
		A=block44,B=block13 -> D = 4;
		A=block44,B=block14 -> D = 3;
		A=block44,B=block20 -> D = 6;
		A=block44,B=block21 -> D = 5;
		A=block44,B=block22 -> D = 4;
		A=block44,B=block23 -> D = 3;
		A=block44,B=block24 -> D = 2;
		A=block44,B=block30 -> D = 5;
		A=block44,B=block31 -> D = 4;
		A=block44,B=block32 -> D = 3;
		A=block44,B=block33 -> D = 2;
		A=block44,B=block34 -> D = 1;
		A=block44,B=block40 -> D = 4;
		A=block44,B=block41 -> D = 3;
		A=block44,B=block42 -> D = 2;
		A=block44,B=block43 -> D = 1;
		D = 0
	)
)
.


%%
%%  prim_obj(Obj,Type):  specify primitive objects in the world
%%
%%  This predicate is true when Obj is the name of a primitive object
%%  in the world, of type Type.
%%
%%  prim_obj(Obj):  shortcut to check object names
%%
%%  This predicate is true if Obj is the name of a primite object,
%%  regardless of its type.
%%

prim_obj(Obj) :-
    prim_obj(Obj,_).

prim_obj(Obj,bowl) :-
    member(Obj,[bowl1,bowl2,bowl3]). 
prim_obj(Obj,pan) :-
    member(Obj,[pan1]).
prim_obj(Obj,board) :-
    member(Obj,[board1,board2]).
prim_obj(Obj,oven) :-
    member(Obj,[oven1]).
prim_obj(Obj,grill) :-
    member(Obj,[grill1]).
prim_obj(Obj,beefstake) :-
    member(Obj,[beefstake1]).
prim_obj(Obj,bacon) :-
    member(Obj,[bacon1]).
prim_obj(Obj,tomato) :-
    member(Obj,[tomato1]).
prim_obj(Obj,lettuce) :-
    member(Obj,[lettuce1]).
prim_obj(Obj,carrot) :-
    member(Obj,[carrot1]).
%% total 25 blocks
prim_obj(Obj,block) :-
    member(Obj,[block00,block01,block02,block03,block04,block10,block11,block12,block13,block14,block20,block21,block22,block23,block24,block30,block31,block32,block33,block34,block40,block41,block42,block43,block44]).
prim_obj(Obj,table) :-
    member(Obj,[table1,table2]).
prim_obj(Obj,sink) :-
    member(Obj,[sink1,sink2,sink3,sink4]).


%%
%%  super_type(SubType,SuperType):  specify type hierarchy
%%
%%  This predicate is true when all objects of type SubType are
%%  also of type SuperType.
%%  
super_type(Type,kitchenware) :-
    member(Type,[oven,grill]).
super_type(Type,utensil) :-
    member(Type,[bowl,board,pan]).
super_type(Type,rawmaterial) :-
    member(Type,[carrot,beefstake,tomato,lettuce,bacon]).
super_type(Type,needcleanstuff):-
    member(Type,[utensil,rawmaterial]).

%%
%%  obj_is_type(Obj,Type):  check object types
%%
%%  This predicate is true when the object named Obj is of type
%%  Type according to the hierarchy of super-types.
%%
obj_is_type(Obj,Type) :-
    prim_obj(Obj,Type)
    ;
    super_type(SubType,Type), obj_is_type(Obj,SubType).


	
%%
%%  prim_action(Act):  specify primitive actions
%%
%%  This predicate is true when Act is the name of a primitive action
%%  in the world.  Actions are typically parameterised in terms of the
%%  objects they act on.  See the details of the MIndiGolog situation
%%  calculus for further information.
%%

%%  acquire_object(Kh,Obj):  kitchenhand acquires control of an object
prim_action(acquire_object(Staff,Obj)) :-
    (kitchenhand(Staff);chef(Staff)), prim_obj(Obj).

%%  release_object(Staff,Obj):  kitchenhand or chef releases an object it has acquired, normally it still contents sth
prim_action(release_object(Staff,Obj)) :-
    (kitchenhand(Staff);chef(Staff)), prim_obj(Obj).
%%  release_clean_object(Staff,Obj):  kitchenhand or chef releases a clean object it has acquired, normally after cleaning
prim_action(release_clean_object(Staff,Obj)) :-
    (kitchenhand(Staff);chef(Staff)), prim_obj(Obj).
%%  release_dirty_object(Staff,Obj):  kitchenhand or chef releases a dirty object it has acquired, normally after use them
prim_action(release_dirty_object(Staff,Obj)) :-
    (kitchenhand(Staff);chef(Staff)), prim_obj(Obj).

%%initial_staff_location(Staff,B):	initial the kitchen hand and chef's initial location
prim_action(initial_staff_location(Staff,B)):-
	(chef(Staff);kitchenhand(Staff)),obj_is_type(B,block).
%%initial_obj_location(Obj,B):	initial the raw material and utensil's initial location
prim_action(initial_obj_location(Obj,B)):-
	prim_obj(Obj),obj_is_type(B,block).

%%  set_timer(Kh,ID,Dur):  kitchenhand or chef sets timer with name ID to ring Dur
%%  minutes in the future
prim_action(set_timer(Staff,_,_)) :-
    chef(Staff);kitchenhand(Staff).

%%  ring_timer(ID):  timer with name ID rings, a natural action
prim_action(ring_timer(_)).
natural(ring_timer(_)).

%%  place_in(Kh,Utens,Dest):  kitchenhand places object Utens in utensil Dest
%%prim_action(place_in(Staff,Utens,Dest)) :-
%%    (kitchenhand(Staff);chef(Staff)), prim_obj(Utens), (obj_is_type(Dest,kitehenware);obj_is_type(Dest,utensil)).
prim_action(place_in(Staff,Obj,Dest)) :-
    (kitchenhand(Staff);chef(Staff)), prim_obj(Obj), (obj_is_type(Dest,kitehenware);obj_is_type(Dest,utensil)).

%%  transfer(Kh,Source,Dest):  kitchenhand transfers contents of utensil Source
%%  to utensil Dest
prim_action(transfer(Staff,Source,Dest)) :-
    (kitchenhand(Staff);chef(Staff)), (obj_is_type(Source,utensil);obj_is_type(Source,kitchenware)),
    (obj_is_type(Dest,utensil);obj_is_type(Dest,kitchenware)).

%%  begin_task(Kh,Task):  kitchenhand starts performing task Task
prim_action(begin_task(Staff,Task)) :-
    (kitchenhand(Staff);chef(Staff)), task(Task).

%%  end_task(Kh,Task):  kitchenhand finishes performing task, a natural action
prim_action(end_task(Staff,Task)) :-
    (kitchenhand(Staff);chef(Staff)), task(Task).
natural(end_task(_,_)).


%%
%%  poss(A,T,S):  possibility of performing an action
%%
%%  This predicate is true when it is possible to perform action
%%  A at time T in situation S.
%%

%%  kitchenhands can only acquire an object if no kitchenhand already has it,
%%  they arent doing a task, and the object hasnt been used.
poss(acquire_object(Kh,Obj),_,S) :-
    \+ has_object(_,Obj,S), \+ doing_task(Kh,_,_,S), \+ used(Obj,S).


%%  kitchenhands may only release objects that they have, when they arent
%%  currently performing a task, the objects could be clean, dirty or still contain something
poss(release_object(Kh,Obj),_,S) :-
    has_object(Kh,Obj,S), \+ doing_task(Kh,_,_,S).

poss(release_clean_object(Kh,Obj),_,S) :-
    has_object(Kh,Obj,S), \+ doing_task(Kh,_,_,S).

poss(release_dirty_object(Kh,Obj),_,S) :-
    has_object(Kh,Obj,S), \+ doing_task(Kh,_,_,S).


poss(initial_staff_location(Staff,_),_,_) :-
	(chef(Staff);kitchenhand(Staff)).
poss(initial_obj_location(Obj,_),_,_) :-
	prim_obj(Obj).


%%  kitchenhands may set a timer as long as it hasnt already been set, and
%%  they arent currently performing a task
poss(set_timer(Kh,ID,_),_,S) :-
    \+ timer_set(ID,_,S), \+ doing_task(Kh,_,_,S).

%%  It is only possible for a timer to ring once its remaining time
%%  has precisely elapsed
poss(ring_timer(ID),T,S) :-
    timer_set(ID,Dur,S),
    start(S,SStart), {T = SStart + Dur}.

%%  kitchenhands may place an object in a utensil provided they have possession
%%  of both, and arent currently doing a task
poss(place_in(Staff,Obj,Dest),_,S) :-
    has_object(Staff,Obj,S), has_object(Staff,Dest,S), is_clean(Dest,S),
    \+ obj_is_type(Obj,kitchenware),
    \+ doing_task(Staff,_,_,S).

%%  kitchenhands may transfer contents from one utensil to another as long
%%  as they have possession of both, and arent doing a task
poss(transfer(Staff,Source,Dest),_,S) :-
    has_object(Staff,Source,S), has_object(Staff,Dest,S), is_clean(Dest,S),
    \+ doing_task(Staff,_,_,S).

%%  kitchenhands may begin the mix() task as long as they arent doing another
%%  task, and have possession of the utensil to be mixed in
poss(begin_task(Kh,mix(Obj,_)),_,S) :-
    has_object(Kh,Obj,S), \+ doing_task(Kh,_,_,S).
poss(begin_task(Kh,marinate(Obj,_)),_,S) :-
    has_object(Kh,Obj,S), \+ doing_task(Kh,_,_,S).
%%  before kitchen hand and chef start goto another block, he has bo be in the orig block first.
poss(begin_task(Kh,goto(A,_)),_,S) :-
    staff_is_in_block(Kh,A,S), \+ doing_task(Kh,_,_,S).


%%  kitchenhands may begin the cut(),clean() task as long as they arent doing another
%%  task, and have possession of the utensil whose contents to cut, clean
poss(begin_task(Kh,cut(Obj,Tb)),_,S) :-
    has_object(Kh,Obj,S),has_object(Kh,Tb,S), \+ doing_task(Kh,_,_,S).

poss(begin_task(Kh,clean(Obj,Sin)),_,S) :-
    has_object(Kh,Sin,S),has_object(Kh,Obj,S), \+ doing_task(Kh,_,_,S).


%%  kitchenhands may end a task only when precisely the remaining amount of
%%  time has elapsed
poss(end_task(Kh,Task),T,S) :-
    doing_task(Kh,Task,Remain,S),
    start(S,SStart), {T = SStart + Remain}.


%%
%%  conflicts(C,T,S):  concurrent actions conflict
%%
%%  This predicate must be true when concurrent actions in C conflict
%%  and cannot be performed at time T in situation S.
%%

%%  kitchenhands cannot do more than one action at a time
conflicts(C,_,_) :-
    member(A1,C), actor(A1,Kh),
    member(A2,C), actor(A2,Kh),
    A2 \= A1.

%%  Two kitchenhands cannot acquire the same object
conflicts(C,_,_) :-
    member(acquire_object(A1,Res),C),
    member(acquire_object(A2,Res),C),
    A1 \= A2.
    

%%  if transfer from utensil O to oven or grill, then the utensil O has to be in the same location of grill and oven
conflicts(C,_,_) :-
    (obj_is_type(S,oven);obj_is_type(S,grill)),	
    member(transfer(O,S),C),
    obj_is_in_block(O,B1),
    obj_is_in_block(S,B2),
    B1 \= B2.

%%  the board and table must be in same block when doing cut
conflicts(C,_,_) :-
    obj_is_type(T,table),	
    member(cut(Bo,T),C),
    obj_is_in_block(T,B1),
    obj_is_in_block(Bo,B2),
    B1 \= B2.

%%  when clean, the object must be in same block with sink
conflicts(C,_,_) :-
    obj_is_type(Sin,sink),	
    member(clean(Obj,Sin),C),
    obj_is_in_block(Sin,B1),
    obj_is_in_block(Obj,B2),
    B1 \= B2.

%%
%%  Fluents in the Domain
%%
%%  The fluents are specified in terms of their successor state axioms,
%%  of the form "a fluent is true if it became true, or was previously
%%  true did not become false".
%%

has_object(Kh,Obj,do(C,T,S)) :-
    member(acquire_object(Kh,Obj),C)
    ;
    has_object(Kh,Obj,S),
    \+ (
       member(release_object(Kh,Obj),C)
       ;
       member(release_dirty_object(Kh,Obj),C)
       ;
       used(Obj,do(C,T,S))
    ).


%%
%%  used(Obj,S):  object is used in situation S
%%
%%  This fluent is true when an object has been used - for example,
%%  an rawmaterial has been placed in a utensil.  Once an object has
%%  been used, it cannot be used again.
%%
used(Obj,do(C,_,S)) :-
    prim_obj(Obj), 
    obj_is_type(Obj,rawmaterial),
    (
      used(Obj,S)
      ;
      member(place_in(_,Obj,_),C)
    ).


%%  is_clean(Obj,S):  object is clean in situation S
%%
%%  This fluent is true when an object has been cleaned
%%

is_clean(Obj,do(C,_,S)) :-
(
	(	
		obj_is_type(Obj,needcleanstuff),
		(
			        member(initial_clean_object(Obj),C)
			;
				member(end_task(_,clean(Obj,_)),C)
		      	;
				(is_clean(Obj,S)
				,
				\+ member(release_dirty_object(_,Obj),C)
				)
		)
	)
	;
	obj_is_type(Obj,kitchenware)
)
.	

%%  is_nearest_obj(Obj,Kh,S):  Kh is the nearest avaliable kitchenhand of Obj
%%

is_nearest_obj(Obj,Kh,do(C,T,S)) :-
(	\+
	(
		prim_obj(Obj),
		kitchenhand(Kh),kitchenhand(Kh1),
		%%both kitchen hand don't have anything, means not in middle of something.
		\+ has_object(Kh,_,do(C,T,S)),\+ has_object(Kh1,_,do(C,T,S)),
		Kh1\=Kh,
		%% Obj is in B_Obj, Kh is in B, Kh1 is in B1
		obj_is_in_block(Obj,B_Obj,do(C,T,S)),
		staff_is_in_block(Kh,B,do(C,T,S)),
		staff_is_in_block(Kh1,B1,do(C,T,S)),
		(M=1;M=2;M=3;M=4;M=5;M=6;M=7;M=8),
		%%Kh1 is nearer to obj than Kh
		(\+ is_near(B,B_Obj,M,do(C,T,S)),is_near(B1,B_Obj,M,do(C,T,S)))
	)
)
.	

%%  is_nearest_kh(Kh,Obj,S):  Obj is the nearest avaliable object of Kh
%%

is_nearest_kh(Kh,Obj,do(C,T,S)) :-
(	\+
	(
		kitchenhand(Kh),
		%% Obj and Obj1 is not equal to each other
		prim_obj(Obj),prim_obj(Obj1),Obj\=Obj1,
		%%Obj and Obj1 is avaliable
		\+ has_object(_,Obj,do(C,T,S)),	\+ has_object(_,Obj1,do(C,T,S)),
		%% Obj and Obj1 is the same type, only sink, table and board 
		%% will be used in this model
		(
			(obj_is_type(Obj,sink),obj_is_type(Obj1,sink));
			(obj_is_type(Obj,table),obj_is_type(Obj1,table));
			(obj_is_type(Obj,board),obj_is_type(Obj1,board))
		),
		%% B,B_Obj,B_Obj1 is the location of Kh, Obj and Obj1		
		staff_is_in_block(Kh,B,do(C,T,S)),
		obj_is_in_block(Obj,B_Obj,do(C,T,S)),
		obj_is_in_block(Obj1,B_Obj1,do(C,T,S)),
		%% Max distance in this domain is 8 
		(M=1;M=2;M=3;M=4;M=5;M=6;M=7;M=8),
		%% Obj1 is nearer to Kh than Obj
		(\+ is_near(B,B_Obj,M,do(C,T,S)),is_near(B,B_Obj1,M,do(C,T,S)))
	)
)
.	

%% A and B is nearer than integer Dist
%% a hash table which is same as task_duration(goto(a,b)) is used in is_near()
is_near(A,B,Dist,do(_,_,_)) :-
(
    ( Dist>0, Dist<9,
	( A=block00,B=block01 , Dist > 1);
	( A=block00,B=block02 , Dist > 2);
	( A=block00,B=block03 , Dist > 3);
	( A=block00,B=block04 , Dist > 4);
	( A=block00,B=block10 , Dist > 1);
	( A=block00,B=block11 , Dist > 2);
	( A=block00,B=block12 , Dist > 3);
	( A=block00,B=block13 , Dist > 4);
	( A=block00,B=block14 , Dist > 5);
	( A=block00,B=block20 , Dist > 2);
	( A=block00,B=block21 , Dist > 3);
	( A=block00,B=block22 , Dist > 4);
	( A=block00,B=block23 , Dist > 5);
	( A=block00,B=block24 , Dist > 6);
	( A=block00,B=block30 , Dist > 3);
	( A=block00,B=block31 , Dist > 4);
	( A=block00,B=block32 , Dist > 5);
	( A=block00,B=block33 , Dist > 6);
	( A=block00,B=block34 , Dist > 7);
	( A=block00,B=block40 , Dist > 4);
	( A=block00,B=block41 , Dist > 5);
	( A=block00,B=block42 , Dist > 6);
	( A=block00,B=block43 , Dist > 7);
	( A=block00,B=block44 , Dist > 8);
	( A=block01,B=block00 , Dist > 1);
	( A=block01,B=block02 , Dist > 1);
	( A=block01,B=block03 , Dist > 2);
	( A=block01,B=block04 , Dist > 3);
	( A=block01,B=block10 , Dist > 2);
	( A=block01,B=block11 , Dist > 1);
	( A=block01,B=block12 , Dist > 2);
	( A=block01,B=block13 , Dist > 3);
	( A=block01,B=block14 , Dist > 4);
	( A=block01,B=block20 , Dist > 3);
	( A=block01,B=block21 , Dist > 2);
	( A=block01,B=block22 , Dist > 3);
	( A=block01,B=block23 , Dist > 4);
	( A=block01,B=block24 , Dist > 5);
	( A=block01,B=block30 , Dist > 4);
	( A=block01,B=block31 , Dist > 3);
	( A=block01,B=block32 , Dist > 4);
	( A=block01,B=block33 , Dist > 5);
	( A=block01,B=block34 , Dist > 6);
	( A=block01,B=block40 , Dist > 5);
	( A=block01,B=block41 , Dist > 4);
	( A=block01,B=block42 , Dist > 5);
	( A=block01,B=block43 , Dist > 6);
	( A=block01,B=block44 , Dist > 7);
	( A=block02,B=block00 , Dist > 2);
	( A=block02,B=block01 , Dist > 1);
	( A=block02,B=block03 , Dist > 1);
	( A=block02,B=block04 , Dist > 2);
	( A=block02,B=block10 , Dist > 3);
	( A=block02,B=block11 , Dist > 2);
	( A=block02,B=block12 , Dist > 1);
	( A=block02,B=block13 , Dist > 2);
	( A=block02,B=block14 , Dist > 3);
	( A=block02,B=block20 , Dist > 4);
	( A=block02,B=block21 , Dist > 3);
	( A=block02,B=block22 , Dist > 2);
	( A=block02,B=block23 , Dist > 3);
	( A=block02,B=block24 , Dist > 4);
	( A=block02,B=block30 , Dist > 5);
	( A=block02,B=block31 , Dist > 4);
	( A=block02,B=block32 , Dist > 3);
	( A=block02,B=block33 , Dist > 4);
	( A=block02,B=block34 , Dist > 5);
	( A=block02,B=block40 , Dist > 6);
	( A=block02,B=block41 , Dist > 5);
	( A=block02,B=block42 , Dist > 4);
	( A=block02,B=block43 , Dist > 5);
	( A=block02,B=block44 , Dist > 6);
	( A=block03,B=block00 , Dist > 3);
	( A=block03,B=block01 , Dist > 2);
	( A=block03,B=block02 , Dist > 1);
	( A=block03,B=block04 , Dist > 1);
	( A=block03,B=block10 , Dist > 4);
	( A=block03,B=block11 , Dist > 3);
	( A=block03,B=block12 , Dist > 2);
	( A=block03,B=block13 , Dist > 1);
	( A=block03,B=block14 , Dist > 2);
	( A=block03,B=block20 , Dist > 5);
	( A=block03,B=block21 , Dist > 4);
	( A=block03,B=block22 , Dist > 3);
	( A=block03,B=block23 , Dist > 2);
	( A=block03,B=block24 , Dist > 3);
	( A=block03,B=block30 , Dist > 6);
	( A=block03,B=block31 , Dist > 5);
	( A=block03,B=block32 , Dist > 4);
	( A=block03,B=block33 , Dist > 3);
	( A=block03,B=block34 , Dist > 4);
	( A=block03,B=block40 , Dist > 7);
	( A=block03,B=block41 , Dist > 6);
	( A=block03,B=block42 , Dist > 5);
	( A=block03,B=block43 , Dist > 4);
	( A=block03,B=block44 , Dist > 5);
	( A=block04,B=block00 , Dist > 4);
	( A=block04,B=block01 , Dist > 3);
	( A=block04,B=block02 , Dist > 2);
	( A=block04,B=block03 , Dist > 1);
	( A=block04,B=block10 , Dist > 5);
	( A=block04,B=block11 , Dist > 4);
	( A=block04,B=block12 , Dist > 3);
	( A=block04,B=block13 , Dist > 2);
	( A=block04,B=block14 , Dist > 1);
	( A=block04,B=block20 , Dist > 6);
	( A=block04,B=block21 , Dist > 5);
	( A=block04,B=block22 , Dist > 4);
	( A=block04,B=block23 , Dist > 3);
	( A=block04,B=block24 , Dist > 2);
	( A=block04,B=block30 , Dist > 7);
	( A=block04,B=block31 , Dist > 6);
	( A=block04,B=block32 , Dist > 5);
	( A=block04,B=block33 , Dist > 4);
	( A=block04,B=block34 , Dist > 3);
	( A=block04,B=block40 , Dist > 8);
	( A=block04,B=block41 , Dist > 7);
	( A=block04,B=block42 , Dist > 6);
	( A=block04,B=block43 , Dist > 5);
	( A=block04,B=block44 , Dist > 4);
	( A=block10,B=block00 , Dist > 1);
	( A=block10,B=block01 , Dist > 2);
	( A=block10,B=block02 , Dist > 3);
	( A=block10,B=block03 , Dist > 4);
	( A=block10,B=block04 , Dist > 5);
	( A=block10,B=block11 , Dist > 1);
	( A=block10,B=block12 , Dist > 2);
	( A=block10,B=block13 , Dist > 3);
	( A=block10,B=block14 , Dist > 4);
	( A=block10,B=block20 , Dist > 1);
	( A=block10,B=block21 , Dist > 2);
	( A=block10,B=block22 , Dist > 3);
	( A=block10,B=block23 , Dist > 4);
	( A=block10,B=block24 , Dist > 5);
	( A=block10,B=block30 , Dist > 2);
	( A=block10,B=block31 , Dist > 3);
	( A=block10,B=block32 , Dist > 4);
	( A=block10,B=block33 , Dist > 5);
	( A=block10,B=block34 , Dist > 6);
	( A=block10,B=block40 , Dist > 3);
	( A=block10,B=block41 , Dist > 4);
	( A=block10,B=block42 , Dist > 5);
	( A=block10,B=block43 , Dist > 6);
	( A=block10,B=block44 , Dist > 7);
	( A=block11,B=block00 , Dist > 2);
	( A=block11,B=block01 , Dist > 1);
	( A=block11,B=block02 , Dist > 2);
	( A=block11,B=block03 , Dist > 3);
	( A=block11,B=block04 , Dist > 4);
	( A=block11,B=block10 , Dist > 1);
	( A=block11,B=block12 , Dist > 1);
	( A=block11,B=block13 , Dist > 2);
	( A=block11,B=block14 , Dist > 3);
	( A=block11,B=block20 , Dist > 2);
	( A=block11,B=block21 , Dist > 1);
	( A=block11,B=block22 , Dist > 2);
	( A=block11,B=block23 , Dist > 3);
	( A=block11,B=block24 , Dist > 4);
	( A=block11,B=block30 , Dist > 3);
	( A=block11,B=block31 , Dist > 2);
	( A=block11,B=block32 , Dist > 3);
	( A=block11,B=block33 , Dist > 4);
	( A=block11,B=block34 , Dist > 5);
	( A=block11,B=block40 , Dist > 4);
	( A=block11,B=block41 , Dist > 3);
	( A=block11,B=block42 , Dist > 4);
	( A=block11,B=block43 , Dist > 5);
	( A=block11,B=block44 , Dist > 6);
	( A=block12,B=block00 , Dist > 3);
	( A=block12,B=block01 , Dist > 2);
	( A=block12,B=block02 , Dist > 1);
	( A=block12,B=block03 , Dist > 2);
	( A=block12,B=block04 , Dist > 3);
	( A=block12,B=block10 , Dist > 2);
	( A=block12,B=block11 , Dist > 1);
	( A=block12,B=block13 , Dist > 1);
	( A=block12,B=block14 , Dist > 2);
	( A=block12,B=block20 , Dist > 3);
	( A=block12,B=block21 , Dist > 2);
	( A=block12,B=block22 , Dist > 1);
	( A=block12,B=block23 , Dist > 2);
	( A=block12,B=block24 , Dist > 3);
	( A=block12,B=block30 , Dist > 4);
	( A=block12,B=block31 , Dist > 3);
	( A=block12,B=block32 , Dist > 2);
	( A=block12,B=block33 , Dist > 3);
	( A=block12,B=block34 , Dist > 4);
	( A=block12,B=block40 , Dist > 5);
	( A=block12,B=block41 , Dist > 4);
	( A=block12,B=block42 , Dist > 3);
	( A=block12,B=block43 , Dist > 4);
	( A=block12,B=block44 , Dist > 5);
	( A=block13,B=block00 , Dist > 4);
	( A=block13,B=block01 , Dist > 3);
	( A=block13,B=block02 , Dist > 2);
	( A=block13,B=block03 , Dist > 1);
	( A=block13,B=block04 , Dist > 2);
	( A=block13,B=block10 , Dist > 3);
	( A=block13,B=block11 , Dist > 2);
	( A=block13,B=block12 , Dist > 1);
	( A=block13,B=block14 , Dist > 1);
	( A=block13,B=block20 , Dist > 4);
	( A=block13,B=block21 , Dist > 3);
	( A=block13,B=block22 , Dist > 2);
	( A=block13,B=block23 , Dist > 1);
	( A=block13,B=block24 , Dist > 2);
	( A=block13,B=block30 , Dist > 5);
	( A=block13,B=block31 , Dist > 4);
	( A=block13,B=block32 , Dist > 3);
	( A=block13,B=block33 , Dist > 2);
	( A=block13,B=block34 , Dist > 3);
	( A=block13,B=block40 , Dist > 6);
	( A=block13,B=block41 , Dist > 5);
	( A=block13,B=block42 , Dist > 4);
	( A=block13,B=block43 , Dist > 3);
	( A=block13,B=block44 , Dist > 4);
	( A=block14,B=block00 , Dist > 5);
	( A=block14,B=block01 , Dist > 4);
	( A=block14,B=block02 , Dist > 3);
	( A=block14,B=block03 , Dist > 2);
	( A=block14,B=block04 , Dist > 1);
	( A=block14,B=block10 , Dist > 4);
	( A=block14,B=block11 , Dist > 3);
	( A=block14,B=block12 , Dist > 2);
	( A=block14,B=block13 , Dist > 1);
	( A=block14,B=block20 , Dist > 5);
	( A=block14,B=block21 , Dist > 4);
	( A=block14,B=block22 , Dist > 3);
	( A=block14,B=block23 , Dist > 2);
	( A=block14,B=block24 , Dist > 1);
	( A=block14,B=block30 , Dist > 6);
	( A=block14,B=block31 , Dist > 5);
	( A=block14,B=block32 , Dist > 4);
	( A=block14,B=block33 , Dist > 3);
	( A=block14,B=block34 , Dist > 2);
	( A=block14,B=block40 , Dist > 7);
	( A=block14,B=block41 , Dist > 6);
	( A=block14,B=block42 , Dist > 5);
	( A=block14,B=block43 , Dist > 4);
	( A=block14,B=block44 , Dist > 3);
	( A=block20,B=block00 , Dist > 2);
	( A=block20,B=block01 , Dist > 3);
	( A=block20,B=block02 , Dist > 4);
	( A=block20,B=block03 , Dist > 5);
	( A=block20,B=block04 , Dist > 6);
	( A=block20,B=block10 , Dist > 1);
	( A=block20,B=block11 , Dist > 2);
	( A=block20,B=block12 , Dist > 3);
	( A=block20,B=block13 , Dist > 4);
	( A=block20,B=block14 , Dist > 5);
	( A=block20,B=block21 , Dist > 1);
	( A=block20,B=block22 , Dist > 2);
	( A=block20,B=block23 , Dist > 3);
	( A=block20,B=block24 , Dist > 4);
	( A=block20,B=block30 , Dist > 1);
	( A=block20,B=block31 , Dist > 2);
	( A=block20,B=block32 , Dist > 3);
	( A=block20,B=block33 , Dist > 4);
	( A=block20,B=block34 , Dist > 5);
	( A=block20,B=block40 , Dist > 2);
	( A=block20,B=block41 , Dist > 3);
	( A=block20,B=block42 , Dist > 4);
	( A=block20,B=block43 , Dist > 5);
	( A=block20,B=block44 , Dist > 6);
	( A=block21,B=block00 , Dist > 3);
	( A=block21,B=block01 , Dist > 2);
	( A=block21,B=block02 , Dist > 3);
	( A=block21,B=block03 , Dist > 4);
	( A=block21,B=block04 , Dist > 5);
	( A=block21,B=block10 , Dist > 2);
	( A=block21,B=block11 , Dist > 1);
	( A=block21,B=block12 , Dist > 2);
	( A=block21,B=block13 , Dist > 3);
	( A=block21,B=block14 , Dist > 4);
	( A=block21,B=block20 , Dist > 1);
	( A=block21,B=block22 , Dist > 1);
	( A=block21,B=block23 , Dist > 2);
	( A=block21,B=block24 , Dist > 3);
	( A=block21,B=block30 , Dist > 2);
	( A=block21,B=block31 , Dist > 1);
	( A=block21,B=block32 , Dist > 2);
	( A=block21,B=block33 , Dist > 3);
	( A=block21,B=block34 , Dist > 4);
	( A=block21,B=block40 , Dist > 3);
	( A=block21,B=block41 , Dist > 2);
	( A=block21,B=block42 , Dist > 3);
	( A=block21,B=block43 , Dist > 4);
	( A=block21,B=block44 , Dist > 5);
	( A=block22,B=block00 , Dist > 4);
	( A=block22,B=block01 , Dist > 3);
	( A=block22,B=block02 , Dist > 2);
	( A=block22,B=block03 , Dist > 3);
	( A=block22,B=block04 , Dist > 4);
	( A=block22,B=block10 , Dist > 3);
	( A=block22,B=block11 , Dist > 2);
	( A=block22,B=block12 , Dist > 1);
	( A=block22,B=block13 , Dist > 2);
	( A=block22,B=block14 , Dist > 3);
	( A=block22,B=block20 , Dist > 2);
	( A=block22,B=block21 , Dist > 1);
	( A=block22,B=block23 , Dist > 1);
	( A=block22,B=block24 , Dist > 2);
	( A=block22,B=block30 , Dist > 3);
	( A=block22,B=block31 , Dist > 2);
	( A=block22,B=block32 , Dist > 1);
	( A=block22,B=block33 , Dist > 2);
	( A=block22,B=block34 , Dist > 3);
	( A=block22,B=block40 , Dist > 4);
	( A=block22,B=block41 , Dist > 3);
	( A=block22,B=block42 , Dist > 2);
	( A=block22,B=block43 , Dist > 3);
	( A=block22,B=block44 , Dist > 4);
	( A=block23,B=block00 , Dist > 5);
	( A=block23,B=block01 , Dist > 4);
	( A=block23,B=block02 , Dist > 3);
	( A=block23,B=block03 , Dist > 2);
	( A=block23,B=block04 , Dist > 3);
	( A=block23,B=block10 , Dist > 4);
	( A=block23,B=block11 , Dist > 3);
	( A=block23,B=block12 , Dist > 2);
	( A=block23,B=block13 , Dist > 1);
	( A=block23,B=block14 , Dist > 2);
	( A=block23,B=block20 , Dist > 3);
	( A=block23,B=block21 , Dist > 2);
	( A=block23,B=block22 , Dist > 1);
	( A=block23,B=block24 , Dist > 1);
	( A=block23,B=block30 , Dist > 4);
	( A=block23,B=block31 , Dist > 3);
	( A=block23,B=block32 , Dist > 2);
	( A=block23,B=block33 , Dist > 1);
	( A=block23,B=block34 , Dist > 2);
	( A=block23,B=block40 , Dist > 5);
	( A=block23,B=block41 , Dist > 4);
	( A=block23,B=block42 , Dist > 3);
	( A=block23,B=block43 , Dist > 2);
	( A=block23,B=block44 , Dist > 3);
	( A=block24,B=block00 , Dist > 6);
	( A=block24,B=block01 , Dist > 5);
	( A=block24,B=block02 , Dist > 4);
	( A=block24,B=block03 , Dist > 3);
	( A=block24,B=block04 , Dist > 2);
	( A=block24,B=block10 , Dist > 5);
	( A=block24,B=block11 , Dist > 4);
	( A=block24,B=block12 , Dist > 3);
	( A=block24,B=block13 , Dist > 2);
	( A=block24,B=block14 , Dist > 1);
	( A=block24,B=block20 , Dist > 4);
	( A=block24,B=block21 , Dist > 3);
	( A=block24,B=block22 , Dist > 2);
	( A=block24,B=block23 , Dist > 1);
	( A=block24,B=block30 , Dist > 5);
	( A=block24,B=block31 , Dist > 4);
	( A=block24,B=block32 , Dist > 3);
	( A=block24,B=block33 , Dist > 2);
	( A=block24,B=block34 , Dist > 1);
	( A=block24,B=block40 , Dist > 6);
	( A=block24,B=block41 , Dist > 5);
	( A=block24,B=block42 , Dist > 4);
	( A=block24,B=block43 , Dist > 3);
	( A=block24,B=block44 , Dist > 2);
	( A=block30,B=block00 , Dist > 3);
	( A=block30,B=block01 , Dist > 4);
	( A=block30,B=block02 , Dist > 5);
	( A=block30,B=block03 , Dist > 6);
	( A=block30,B=block04 , Dist > 7);
	( A=block30,B=block10 , Dist > 2);
	( A=block30,B=block11 , Dist > 3);
	( A=block30,B=block12 , Dist > 4);
	( A=block30,B=block13 , Dist > 5);
	( A=block30,B=block14 , Dist > 6);
	( A=block30,B=block20 , Dist > 1);
	( A=block30,B=block21 , Dist > 2);
	( A=block30,B=block22 , Dist > 3);
	( A=block30,B=block23 , Dist > 4);
	( A=block30,B=block24 , Dist > 5);
	( A=block30,B=block31 , Dist > 1);
	( A=block30,B=block32 , Dist > 2);
	( A=block30,B=block33 , Dist > 3);
	( A=block30,B=block34 , Dist > 4);
	( A=block30,B=block40 , Dist > 1);
	( A=block30,B=block41 , Dist > 2);
	( A=block30,B=block42 , Dist > 3);
	( A=block30,B=block43 , Dist > 4);
	( A=block30,B=block44 , Dist > 5);
	( A=block31,B=block00 , Dist > 4);
	( A=block31,B=block01 , Dist > 3);
	( A=block31,B=block02 , Dist > 4);
	( A=block31,B=block03 , Dist > 5);
	( A=block31,B=block04 , Dist > 6);
	( A=block31,B=block10 , Dist > 3);
	( A=block31,B=block11 , Dist > 2);
	( A=block31,B=block12 , Dist > 3);
	( A=block31,B=block13 , Dist > 4);
	( A=block31,B=block14 , Dist > 5);
	( A=block31,B=block20 , Dist > 2);
	( A=block31,B=block21 , Dist > 1);
	( A=block31,B=block22 , Dist > 2);
	( A=block31,B=block23 , Dist > 3);
	( A=block31,B=block24 , Dist > 4);
	( A=block31,B=block30 , Dist > 1);
	( A=block31,B=block32 , Dist > 1);
	( A=block31,B=block33 , Dist > 2);
	( A=block31,B=block34 , Dist > 3);
	( A=block31,B=block40 , Dist > 2);
	( A=block31,B=block41 , Dist > 1);
	( A=block31,B=block42 , Dist > 2);
	( A=block31,B=block43 , Dist > 3);
	( A=block31,B=block44 , Dist > 4);
	( A=block32,B=block00 , Dist > 5);
	( A=block32,B=block01 , Dist > 4);
	( A=block32,B=block02 , Dist > 3);
	( A=block32,B=block03 , Dist > 4);
	( A=block32,B=block04 , Dist > 5);
	( A=block32,B=block10 , Dist > 4);
	( A=block32,B=block11 , Dist > 3);
	( A=block32,B=block12 , Dist > 2);
	( A=block32,B=block13 , Dist > 3);
	( A=block32,B=block14 , Dist > 4);
	( A=block32,B=block20 , Dist > 3);
	( A=block32,B=block21 , Dist > 2);
	( A=block32,B=block22 , Dist > 1);
	( A=block32,B=block23 , Dist > 2);
	( A=block32,B=block24 , Dist > 3);
	( A=block32,B=block30 , Dist > 2);
	( A=block32,B=block31 , Dist > 1);
	( A=block32,B=block33 , Dist > 1);
	( A=block32,B=block34 , Dist > 2);
	( A=block32,B=block40 , Dist > 3);
	( A=block32,B=block41 , Dist > 2);
	( A=block32,B=block42 , Dist > 1);
	( A=block32,B=block43 , Dist > 2);
	( A=block32,B=block44 , Dist > 3);
	( A=block33,B=block00 , Dist > 6);
	( A=block33,B=block01 , Dist > 5);
	( A=block33,B=block02 , Dist > 4);
	( A=block33,B=block03 , Dist > 3);
	( A=block33,B=block04 , Dist > 4);
	( A=block33,B=block10 , Dist > 5);
	( A=block33,B=block11 , Dist > 4);
	( A=block33,B=block12 , Dist > 3);
	( A=block33,B=block13 , Dist > 2);
	( A=block33,B=block14 , Dist > 3);
	( A=block33,B=block20 , Dist > 4);
	( A=block33,B=block21 , Dist > 3);
	( A=block33,B=block22 , Dist > 2);
	( A=block33,B=block23 , Dist > 1);
	( A=block33,B=block24 , Dist > 2);
	( A=block33,B=block30 , Dist > 3);
	( A=block33,B=block31 , Dist > 2);
	( A=block33,B=block32 , Dist > 1);
	( A=block33,B=block34 , Dist > 1);
	( A=block33,B=block40 , Dist > 4);
	( A=block33,B=block41 , Dist > 3);
	( A=block33,B=block42 , Dist > 2);
	( A=block33,B=block43 , Dist > 1);
	( A=block33,B=block44 , Dist > 2);
	( A=block34,B=block00 , Dist > 7);
	( A=block34,B=block01 , Dist > 6);
	( A=block34,B=block02 , Dist > 5);
	( A=block34,B=block03 , Dist > 4);
	( A=block34,B=block04 , Dist > 3);
	( A=block34,B=block10 , Dist > 6);
	( A=block34,B=block11 , Dist > 5);
	( A=block34,B=block12 , Dist > 4);
	( A=block34,B=block13 , Dist > 3);
	( A=block34,B=block14 , Dist > 2);
	( A=block34,B=block20 , Dist > 5);
	( A=block34,B=block21 , Dist > 4);
	( A=block34,B=block22 , Dist > 3);
	( A=block34,B=block23 , Dist > 2);
	( A=block34,B=block24 , Dist > 1);
	( A=block34,B=block30 , Dist > 4);
	( A=block34,B=block31 , Dist > 3);
	( A=block34,B=block32 , Dist > 2);
	( A=block34,B=block33 , Dist > 1);
	( A=block34,B=block40 , Dist > 5);
	( A=block34,B=block41 , Dist > 4);
	( A=block34,B=block42 , Dist > 3);
	( A=block34,B=block43 , Dist > 2);
	( A=block34,B=block44 , Dist > 1);
	( A=block40,B=block00 , Dist > 4);
	( A=block40,B=block01 , Dist > 5);
	( A=block40,B=block02 , Dist > 6);
	( A=block40,B=block03 , Dist > 7);
	( A=block40,B=block04 , Dist > 8);
	( A=block40,B=block10 , Dist > 3);
	( A=block40,B=block11 , Dist > 4);
	( A=block40,B=block12 , Dist > 5);
	( A=block40,B=block13 , Dist > 6);
	( A=block40,B=block14 , Dist > 7);
	( A=block40,B=block20 , Dist > 2);
	( A=block40,B=block21 , Dist > 3);
	( A=block40,B=block22 , Dist > 4);
	( A=block40,B=block23 , Dist > 5);
	( A=block40,B=block24 , Dist > 6);
	( A=block40,B=block30 , Dist > 1);
	( A=block40,B=block31 , Dist > 2);
	( A=block40,B=block32 , Dist > 3);
	( A=block40,B=block33 , Dist > 4);
	( A=block40,B=block34 , Dist > 5);
	( A=block40,B=block41 , Dist > 1);
	( A=block40,B=block42 , Dist > 2);
	( A=block40,B=block43 , Dist > 3);
	( A=block40,B=block44 , Dist > 4);
	( A=block41,B=block00 , Dist > 5);
	( A=block41,B=block01 , Dist > 4);
	( A=block41,B=block02 , Dist > 5);
	( A=block41,B=block03 , Dist > 6);
	( A=block41,B=block04 , Dist > 7);
	( A=block41,B=block10 , Dist > 4);
	( A=block41,B=block11 , Dist > 3);
	( A=block41,B=block12 , Dist > 4);
	( A=block41,B=block13 , Dist > 5);
	( A=block41,B=block14 , Dist > 6);
	( A=block41,B=block20 , Dist > 3);
	( A=block41,B=block21 , Dist > 2);
	( A=block41,B=block22 , Dist > 3);
	( A=block41,B=block23 , Dist > 4);
	( A=block41,B=block24 , Dist > 5);
	( A=block41,B=block30 , Dist > 2);
	( A=block41,B=block31 , Dist > 1);
	( A=block41,B=block32 , Dist > 2);
	( A=block41,B=block33 , Dist > 3);
	( A=block41,B=block34 , Dist > 4);
	( A=block41,B=block40 , Dist > 1);
	( A=block41,B=block42 , Dist > 1);
	( A=block41,B=block43 , Dist > 2);
	( A=block41,B=block44 , Dist > 3);
	( A=block42,B=block00 , Dist > 6);
	( A=block42,B=block01 , Dist > 5);
	( A=block42,B=block02 , Dist > 4);
	( A=block42,B=block03 , Dist > 5);
	( A=block42,B=block04 , Dist > 6);
	( A=block42,B=block10 , Dist > 5);
	( A=block42,B=block11 , Dist > 4);
	( A=block42,B=block12 , Dist > 3);
	( A=block42,B=block13 , Dist > 4);
	( A=block42,B=block14 , Dist > 5);
	( A=block42,B=block20 , Dist > 4);
	( A=block42,B=block21 , Dist > 3);
	( A=block42,B=block22 , Dist > 2);
	( A=block42,B=block23 , Dist > 3);
	( A=block42,B=block24 , Dist > 4);
	( A=block42,B=block30 , Dist > 3);
	( A=block42,B=block31 , Dist > 2);
	( A=block42,B=block32 , Dist > 1);
	( A=block42,B=block33 , Dist > 2);
	( A=block42,B=block34 , Dist > 3);
	( A=block42,B=block40 , Dist > 2);
	( A=block42,B=block41 , Dist > 1);
	( A=block42,B=block43 , Dist > 1);
	( A=block42,B=block44 , Dist > 2);
	( A=block43,B=block00 , Dist > 7);
	( A=block43,B=block01 , Dist > 6);
	( A=block43,B=block02 , Dist > 5);
	( A=block43,B=block03 , Dist > 4);
	( A=block43,B=block04 , Dist > 5);
	( A=block43,B=block10 , Dist > 6);
	( A=block43,B=block11 , Dist > 5);
	( A=block43,B=block12 , Dist > 4);
	( A=block43,B=block13 , Dist > 3);
	( A=block43,B=block14 , Dist > 4);
	( A=block43,B=block20 , Dist > 5);
	( A=block43,B=block21 , Dist > 4);
	( A=block43,B=block22 , Dist > 3);
	( A=block43,B=block23 , Dist > 2);
	( A=block43,B=block24 , Dist > 3);
	( A=block43,B=block30 , Dist > 4);
	( A=block43,B=block31 , Dist > 3);
	( A=block43,B=block32 , Dist > 2);
	( A=block43,B=block33 , Dist > 1);
	( A=block43,B=block34 , Dist > 2);
	( A=block43,B=block40 , Dist > 3);
	( A=block43,B=block41 , Dist > 2);
	( A=block43,B=block42 , Dist > 1);
	( A=block43,B=block44 , Dist > 1);
	( A=block44,B=block00 , Dist > 8);
	( A=block44,B=block01 , Dist > 7);
	( A=block44,B=block02 , Dist > 6);
	( A=block44,B=block03 , Dist > 5);
	( A=block44,B=block04 , Dist > 4);
	( A=block44,B=block10 , Dist > 7);
	( A=block44,B=block11 , Dist > 6);
	( A=block44,B=block12 , Dist > 5);
	( A=block44,B=block13 , Dist > 4);
	( A=block44,B=block14 , Dist > 3);
	( A=block44,B=block20 , Dist > 6);
	( A=block44,B=block21 , Dist > 5);
	( A=block44,B=block22 , Dist > 4);
	( A=block44,B=block23 , Dist > 3);
	( A=block44,B=block24 , Dist > 2);
	( A=block44,B=block30 , Dist > 5);
	( A=block44,B=block31 , Dist > 4);
	( A=block44,B=block32 , Dist > 3);
	( A=block44,B=block33 , Dist > 2);
	( A=block44,B=block34 , Dist > 1);
	( A=block44,B=block40 , Dist > 4);
	( A=block44,B=block41 , Dist > 3);
	( A=block44,B=block42 , Dist > 2);
	( A=block44,B=block43 , Dist > 1);
	( A=B)	
    )
).

%%
%%  timer_set(ID,Dur,S):  timer is set in situation S
%%
%%  This fluent indicates that the timer named ID is set to ring in Dur
%%  minutes in situation S.  It becomes true as a result of a set_timer()
%%  action, and is updated from situation to situation to reflect
%%  the amount of time remaining.
%%
timer_set(ID,Dur,do(C,T,S)) :-
    member(set_timer(_,ID,Dur),C)
    ;
    timer_set(ID,OldDur,S), start(S,SStart), {Dur = OldDur-(T-SStart)},
    \+ member(ring_timer(ID),C).



%%  staff_is_in_block(Staff,B,S):  Staff is in block B
%%
%%  This fluent is true staff is in block B
%%
staff_is_in_block(Staff,B,do(C,_,S)) :-
	%%Staff is initialled in block B
	(member(initial_staff_location(Staff,B),C))
	;
	%%Staff just go into block B
	(
	member(end_task(Staff,goto(_,B)),C)
	)
	;
	%%Staff is in block B before and not go out block B
	(
	staff_is_in_block(Staff,B,S),
	\+ member(begin_task(Staff,goto(B,_)),C)
	)
    .


%%  obj_is_in_block(Obj,B,S):  Obj is in block B
%%
%%  This fluent is true Obj is in block B
%%
obj_is_in_block(Obj,B,do(C,_,S)) :-
	(
		(obj_is_type(Obj,rawmaterial);obj_is_type(Obj,utensil)),
		(
			%%Obj is initialled in block B
			(member(initial_obj_location(Obj,B),C))
			;
			%%Obj is taken by Staff and Staff go to block B
			(
			member(end_task(Staff,goto(_,B)),C),has_object(Staff,Obj,S)
			)
			;
			%%Obj is in block B before and no Staff takes it out that block
			(
			obj_is_in_block(Obj,B,S),
			\+ (member(begin_task(Staff,goto(B,_)),C),has_object(Staff,Obj,S))
			)
		)
	)
	;
	(
		(obj_is_type(Obj,kitchen_ware);obj_is_type(Obj,table);obj_is_type(Obj,sink)),
		(
			%%Obj is initialled in block B
			member(initial_obj_location(Obj,B),C)
			;
			%%Obj is in block B and always be there.
			obj_is_in_block(Obj,B,S)
		)
	)
    .

%%
%%  doing_task(Kh,Task,Remain,S):  kitchenhand is performing a task
%%
%%  This fluent is true when kitchenhand Kh is performing task Task in situation
%%  S, and has Remain minutes left before it is completed.  It becomes
%%  true when an kitchenhand begins a task, becomes false when an kitchenhand
%%  completes a task, and its value is updated from situation to situation
%%  to reflect the remianing time till completion.
%%
doing_task(Kh,Task,Remain,do(C,T,S)) :-
    member(begin_task(Kh,Task),C), task_duration(Kh,Task,Remain)
    ;
    doing_task(Kh,Task,OldRem,S), start(S,SStart),
    {OldRem = Remain-(T-SStart)}, \+ member(end_task(Kh,Task),C).
    
    

%%
%%  history_length(N,S):  length of the action histoy in a situation
%%
%%  This simple fluent encodes in N the number of actions that have
%%  taken place in the history of situation S.  It is used to make this
%%  information easily available to kitchenhands.
%%
history_length(N,do(_,_,S)) :-
    history_length(N1,S),
    N is N1 + 1.
history_length(0,s0).

%%
%%  Intial Conditions for the domain
%%
%%  The initial conditions are specified by additional clauses for
%%  each fluent, with the situation term set to the atom s0.  For
%%  the most part no fluents hold in the initial situation, so 
%%  there arent many clauses here.
%%

start(s0,0).



